@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -45,7 +45,6 @@ $fields = [
 	'key' =>					[T_ZBX_STR, O_OPT, null,	NOT_EMPTY, 'isset({add}) || isset({update})', _('Key')],
 	'master_itemid' =>			[T_ZBX_STR, O_OPT, null,	null,
 		'(isset({add}) || isset({update})) && isset({type}) && {type}=='.ITEM_TYPE_DEPENDENT, _('Master item')],
-	'master_itemname' =>		[T_ZBX_STR, O_OPT, null,	null,			null],
 	'delay' =>					[T_ZBX_TU, O_OPT, P_ALLOW_USER_MACRO, null,
 		'(isset({add}) || isset({update})) && isset({type}) && {type}!='.ITEM_TYPE_TRAPPER.
 			' && {type}!='.ITEM_TYPE_SNMPTRAP.' && {type}!='.ITEM_TYPE_DEPENDENT,
@@ -57,9 +56,15 @@ $fields = [
 	],
 	'status' =>					[T_ZBX_INT, O_OPT, null,	IN([ITEM_STATUS_DISABLED, ITEM_STATUS_ACTIVE]), null],
 	'type' =>					[T_ZBX_INT, O_OPT, null,
-		IN([-1, ITEM_TYPE_ZABBIX, ITEM_TYPE_SNMPV1, ITEM_TYPE_TRAPPER, ITEM_TYPE_SIMPLE, ITEM_TYPE_SNMPV2C,
-			ITEM_TYPE_INTERNAL, ITEM_TYPE_SNMPV3, ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL,
-			ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT]), 'isset({add}) || isset({update})'],
+									IN([-1, ITEM_TYPE_ZABBIX, ITEM_TYPE_SNMPV1, ITEM_TYPE_TRAPPER, ITEM_TYPE_SIMPLE,
+										ITEM_TYPE_SNMPV2C, ITEM_TYPE_INTERNAL, ITEM_TYPE_SNMPV3,
+										ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL,
+										ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET,
+										ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT,
+										ITEM_TYPE_HTTPAGENT
+									]),
+									'isset({add}) || isset({update})'
+								],
 	'trends' =>					[T_ZBX_STR, O_OPT, null,	null,
 		'(isset({add}) || isset({update})) && isset({value_type}) && '.
 			IN(ITEM_VALUE_TYPE_FLOAT.','.ITEM_VALUE_TYPE_UINT64, 'value_type'),
@@ -125,10 +130,70 @@ $fields = [
 	'jmx_endpoint' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
 		'(isset({add}) || isset({update})) && isset({type}) && {type} == '.ITEM_TYPE_JMX
 	],
+	'timeout' => 				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'url' =>            		[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
+		'(isset({add}) || isset({update})) && isset({type}) && {type} == '.ITEM_TYPE_HTTPAGENT, _('URL')],
+	'query_fields' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'posts' =>					[T_ZBX_STR, O_OPT, null,	null,		null],
+	'status_codes' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'follow_redirects' =>		[T_ZBX_INT, O_OPT, null,
+									IN([HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF, HTTPTEST_STEP_FOLLOW_REDIRECTS_ON]),
+									null
+								],
+	'post_type' =>				[T_ZBX_INT, O_OPT, null,
+									IN([ZBX_POSTTYPE_RAW, ZBX_POSTTYPE_JSON, ZBX_POSTTYPE_XML]),
+									null
+								],
+	'http_proxy' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'headers' => 				[T_ZBX_STR, O_OPT, null,	null,		null],
+	'retrieve_mode' =>			[T_ZBX_INT, O_OPT, null,
+									IN([HTTPTEST_STEP_RETRIEVE_MODE_CONTENT, HTTPTEST_STEP_RETRIEVE_MODE_HEADERS,
+										HTTPTEST_STEP_RETRIEVE_MODE_BOTH
+									]),
+									null
+								],
+	'request_method' =>			[T_ZBX_INT, O_OPT, null,
+									IN([HTTPCHECK_REQUEST_GET, HTTPCHECK_REQUEST_POST, HTTPCHECK_REQUEST_PUT,
+										HTTPCHECK_REQUEST_HEAD
+									]),
+									null
+								],
+	'output_format' =>			[T_ZBX_INT, O_OPT, null,	IN([HTTPCHECK_STORE_RAW, HTTPCHECK_STORE_JSON]), null],
+	'allow_traps' =>			[T_ZBX_INT, O_OPT, null,	IN([HTTPCHECK_ALLOW_TRAPS_OFF, HTTPCHECK_ALLOW_TRAPS_ON]),
+									null
+								],
+	'ssl_cert_file' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'ssl_key_file' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
+	'ssl_key_password' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
+	'verify_peer' =>			[T_ZBX_INT, O_OPT, null,
+									IN([HTTPTEST_VERIFY_PEER_OFF, HTTPTEST_VERIFY_PEER_ON]),
+									null
+								],
+	'verify_host' =>			[T_ZBX_INT, O_OPT, null,
+									IN([HTTPTEST_VERIFY_HOST_OFF, HTTPTEST_VERIFY_HOST_ON]),
+									null
+								],
+	'http_authtype' =>			[T_ZBX_INT, O_OPT, null,
+									IN([HTTPTEST_AUTH_NONE, HTTPTEST_AUTH_BASIC, HTTPTEST_AUTH_NTLM]),
+									null
+								],
+	'http_username' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
+									'(isset({add}) || isset({update})) && isset({http_authtype})'.
+										' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
+											' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.')',
+									_('Username')
+								],
+	'http_password' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,
+									'(isset({add}) || isset({update})) && isset({http_authtype})'.
+										' && ({http_authtype} == '.HTTPTEST_AUTH_BASIC.
+											' || {http_authtype} == '.HTTPTEST_AUTH_NTLM.')',
+									_('Password')
+								],
 	// actions
 	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 									IN('"item.massclearhistory","item.masscopyto","item.massdelete",'.
-										'"item.massdisable","item.massenable","item.massupdateform"'
+										'"item.massdisable","item.massenable","item.massupdateform",'.
+										'"item.masscheck_now"'
 									),
 									null
 								],
@@ -139,6 +204,7 @@ $fields = [
 	'delete' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'massupdate' =>				[T_ZBX_STR, O_OPT, P_SYS, null,	null],
 	'cancel' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'check_now' =>				[T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,	null],
 	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form_refresh' =>			[T_ZBX_INT, O_OPT, null,	null,		null],
 	// filter
@@ -149,10 +215,15 @@ $fields = [
 	'filter_application' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
 	'filter_name' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
 	'filter_type' =>			[T_ZBX_INT, O_OPT, null,
-		IN([-1, ITEM_TYPE_ZABBIX, ITEM_TYPE_SNMPV1, ITEM_TYPE_TRAPPER, ITEM_TYPE_SIMPLE, ITEM_TYPE_SNMPV2C,
-		ITEM_TYPE_INTERNAL, ITEM_TYPE_SNMPV3, ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR,
-		ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMPTRAP,
-		ITEM_TYPE_DEPENDENT]), null],
+									IN([-1, ITEM_TYPE_ZABBIX, ITEM_TYPE_SNMPV1, ITEM_TYPE_TRAPPER, ITEM_TYPE_SIMPLE,
+										ITEM_TYPE_SNMPV2C, ITEM_TYPE_INTERNAL, ITEM_TYPE_SNMPV3,
+										ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_AGGREGATE, ITEM_TYPE_EXTERNAL,
+										ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET,
+										ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT,
+										ITEM_TYPE_HTTPAGENT
+									]),
+									null
+								],
 	'filter_key' =>				[T_ZBX_STR, O_OPT, null,	null,		null],
 	'filter_snmp_community' =>	[T_ZBX_STR, O_OPT, null,	null,		null],
 	'filter_snmpv3_securityname' => [T_ZBX_STR, O_OPT, null, null,		null],
@@ -166,6 +237,7 @@ $fields = [
 	'filter_state' =>			[T_ZBX_INT, O_OPT, null,	IN([-1, ITEM_STATE_NORMAL, ITEM_STATE_NOTSUPPORTED]), null],
 	'filter_templated_items' => [T_ZBX_INT, O_OPT, null,	IN('-1,0,1'), null],
 	'filter_with_triggers' =>	[T_ZBX_INT, O_OPT, null,	IN('-1,0,1'), null],
+	'filter_discovery' =>		[T_ZBX_INT, O_OPT, null,	IN([-1, ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]), null],
 	'filter_ipmi_sensor' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
 	// subfilters
 	'subfilter_set' =>			[T_ZBX_STR, O_OPT, null,	null,		null],
@@ -176,6 +248,7 @@ $fields = [
 	'subfilter_state' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
 	'subfilter_templated_items' => [T_ZBX_INT, O_OPT, null, null,		null],
 	'subfilter_with_triggers' => [T_ZBX_INT, O_OPT, null,	null,		null],
+	'subfilter_discovery' =>	[T_ZBX_INT, O_OPT, null,	null,		null],
 	'subfilter_hosts' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
 	'subfilter_interval' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
 	'subfilter_history' =>		[T_ZBX_STR, O_OPT, null,	null,		null],
@@ -195,7 +268,7 @@ unset($_REQUEST[$paramsFieldName]);
 
 $subfiltersList = ['subfilter_apps', 'subfilter_types', 'subfilter_value_types', 'subfilter_status',
 	'subfilter_state', 'subfilter_templated_items', 'subfilter_with_triggers', 'subfilter_hosts', 'subfilter_interval',
-	'subfilter_history', 'subfilter_trends'
+	'subfilter_history', 'subfilter_trends', 'subfilter_discovery'
 ];
 
 /*
@@ -306,6 +379,7 @@ if (hasRequest('filter_set')) {
 	CProfile::update('web.items.filter_state', getRequest('filter_state', -1), PROFILE_TYPE_INT);
 	CProfile::update('web.items.filter_templated_items', getRequest('filter_templated_items', -1), PROFILE_TYPE_INT);
 	CProfile::update('web.items.filter_with_triggers', getRequest('filter_with_triggers', -1), PROFILE_TYPE_INT);
+	CProfile::update('web.items.filter_discovery', getRequest('filter_discovery', -1), PROFILE_TYPE_INT);
 	CProfile::update('web.items.filter_ipmi_sensor', getRequest('filter_ipmi_sensor', ''), PROFILE_TYPE_STR);
 
 	// subfilters
@@ -354,6 +428,7 @@ $_REQUEST['filter_trends'] = CProfile::get('web.items.filter_trends', '');
 $_REQUEST['filter_status'] = CProfile::get('web.items.filter_status', -1);
 $_REQUEST['filter_state'] = CProfile::get('web.items.filter_state', -1);
 $_REQUEST['filter_templated_items'] = CProfile::get('web.items.filter_templated_items', -1);
+$_REQUEST['filter_discovery'] = CProfile::get('web.items.filter_discovery', -1);
 $_REQUEST['filter_with_triggers'] = CProfile::get('web.items.filter_with_triggers', -1);
 $_REQUEST['filter_ipmi_sensor'] = CProfile::get('web.items.filter_ipmi_sensor', '');
 
@@ -447,6 +522,9 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	if (!in_array($type, [ITEM_TYPE_ZABBIX_ACTIVE, ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP])
 			&& hasRequest('delay_flex')) {
 		$intervals = [];
+		$simple_interval_parser = new CSimpleIntervalParser(['usermacros' => true]);
+		$time_period_parser = new CTimePeriodParser(['usermacros' => true]);
+		$scheduling_interval_parser = new CSchedulingIntervalParser(['usermacros' => true]);
 
 		foreach (getRequest('delay_flex') as $interval) {
 			if ($interval['type'] == ITEM_DELAY_FLEXIBLE) {
@@ -454,12 +532,12 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					continue;
 				}
 
-				if (strpos($interval['delay'], ';') !== false) {
+				if ($simple_interval_parser->parse($interval['delay']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['delay']));
 					break;
 				}
-				elseif (strpos($interval['period'], ';') !== false) {
+				elseif ($time_period_parser->parse($interval['period']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['period']));
 					break;
@@ -472,7 +550,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					continue;
 				}
 
-				if (strpos($interval['schedule'], ';') !== false) {
+				if ($scheduling_interval_parser->parse($interval['schedule']) != CParser::PARSE_SUCCESS) {
 					$result = false;
 					info(_s('Invalid interval "%1$s".', $interval['schedule']));
 					break;
@@ -546,9 +624,36 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				'applications' => $applications,
 				'inventory_link' => getRequest('inventory_link', 0),
 				'description' => getRequest('description', ''),
-				'status' => getRequest('status', ITEM_STATUS_DISABLED)
+				'status' => getRequest('status', ITEM_STATUS_DISABLED),
 			];
 
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				$http_item = [
+					'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
+					'url' => getRequest('url'),
+					'query_fields' => getRequest('query_fields', []),
+					'posts' => getRequest('posts'),
+					'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
+					'follow_redirects' => (int) getRequest('follow_redirects'),
+					'post_type' => (int) getRequest('post_type'),
+					'http_proxy' => getRequest('http_proxy'),
+					'headers' => getRequest('headers', []),
+					'retrieve_mode' => (int) getRequest('retrieve_mode'),
+					'request_method' => (int) getRequest('request_method'),
+					'output_format' => (int) getRequest('output_format'),
+					'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
+					'ssl_cert_file' => getRequest('ssl_cert_file'),
+					'ssl_key_file' => getRequest('ssl_key_file'),
+					'ssl_key_password' => getRequest('ssl_key_password'),
+					'verify_peer' => (int) getRequest('verify_peer'),
+					'verify_host' => (int) getRequest('verify_host'),
+					'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
+					'username' => getRequest('http_username', ''),
+					'password' => getRequest('http_password', '')
+				];
+				$item = prepareItemHttpAgentFormData($http_item) + $item;
+			}
+
 			if ($item['type'] == ITEM_TYPE_JMX) {
 				$item['jmx_endpoint'] = getRequest('jmx_endpoint', '');
 			}
@@ -570,7 +675,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 					'snmpv3_privprotocol', 'snmpv3_privpassphrase', 'port', 'authtype', 'username', 'password',
 					'publickey', 'privatekey', 'params', 'ipmi_sensor', 'value_type', 'units', 'delay', 'history',
 					'trends', 'valuemapid', 'logtimefmt', 'trapper_hosts', 'inventory_link', 'description', 'status',
-					'templateid', 'flags', 'jmx_endpoint', 'master_itemid'
+					'templateid', 'flags', 'jmx_endpoint', 'master_itemid', 'timeout', 'url', 'query_fields', 'posts',
+					'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode',
+					'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
+					'verify_peer', 'verify_host', 'allow_traps'
 				],
 				'selectApplications' => ['applicationid'],
 				'selectPreprocessing' => ['type', 'params'],
@@ -696,6 +804,32 @@ elseif (hasRequest('add') || hasRequest('update')) {
 				if ($db_item['preprocessing'] !== $preprocessing) {
 					$item['preprocessing'] = $preprocessing;
 				}
+				if (getRequest('type') == ITEM_TYPE_HTTPAGENT) {
+					$http_item = [
+						'timeout' => getRequest('timeout', DB::getDefault('items', 'timeout')),
+						'url' => getRequest('url'),
+						'query_fields' => getRequest('query_fields', []),
+						'posts' => getRequest('posts'),
+						'status_codes' => getRequest('status_codes', DB::getDefault('items', 'status_codes')),
+						'follow_redirects' => (int) getRequest('follow_redirects'),
+						'post_type' => (int) getRequest('post_type'),
+						'http_proxy' => getRequest('http_proxy'),
+						'headers' => getRequest('headers', []),
+						'retrieve_mode' => (int) getRequest('retrieve_mode'),
+						'request_method' => (int) getRequest('request_method'),
+						'output_format' => (int) getRequest('output_format'),
+						'allow_traps' => (int) getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF),
+						'ssl_cert_file' => getRequest('ssl_cert_file'),
+						'ssl_key_file' => getRequest('ssl_key_file'),
+						'ssl_key_password' => getRequest('ssl_key_password'),
+						'verify_peer' => (int) getRequest('verify_peer'),
+						'verify_host' => (int) getRequest('verify_host'),
+						'authtype' => getRequest('http_authtype', HTTPTEST_AUTH_NONE),
+						'username' => getRequest('http_username', ''),
+						'password' => getRequest('http_password', '')
+					];
+					$item = prepareItemHttpAgentFormData($http_item) + $item;
+				}
 			}
 
 			if ($db_item['status'] != getRequest('status', ITEM_STATUS_DISABLED)) {
@@ -732,6 +866,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		uncheckTableRows(getRequest('hostid'));
 	}
 }
+elseif (hasRequest('check_now') && hasRequest('itemid')) {
+	$result = (bool) API::Task()->create([
+		'type' => ZBX_TM_TASK_CHECK_NOW,
+		'itemids' => getRequest('itemid')
+	]);
+
+	show_messages($result, _('Request sent successfully'), _('Cannot send request'));
+}
 // cleaning history for one item
 elseif (hasRequest('del_history') && hasRequest('itemid')) {
 	$result = false;
@@ -771,11 +913,14 @@ elseif ($valid_input && hasRequest('massupdate') && hasRequest('group_itemid'))
 
 	$result = true;
 
-	if (isset($visible['update_interval'])) {
+	if (isset($visible['delay'])) {
 		$delay = getRequest('delay', DB::getDefault('items', 'delay'));
 
 		if (hasRequest('delay_flex')) {
 			$intervals = [];
+			$simple_interval_parser = new CSimpleIntervalParser(['usermacros' => true]);
+			$time_period_parser = new CTimePeriodParser(['usermacros' => true]);
+			$scheduling_interval_parser = new CSchedulingIntervalParser(['usermacros' => true]);
 
 			foreach (getRequest('delay_flex') as $interval) {
 				if ($interval['type'] == ITEM_DELAY_FLEXIBLE) {
@@ -783,12 +928,12 @@ elseif ($valid_input && hasRequest('massupdate') && hasRequest('group_itemid'))
 						continue;
 					}
 
-					if (strpos($interval['delay'], ';') !== false) {
+					if ($simple_interval_parser->parse($interval['delay']) != CParser::PARSE_SUCCESS) {
 						$result = false;
 						info(_s('Invalid interval "%1$s".', $interval['delay']));
 						break;
 					}
-					elseif (strpos($interval['period'], ';')  !== false) {
+					elseif ($time_period_parser->parse($interval['period']) != CParser::PARSE_SUCCESS) {
 						$result = false;
 						info(_s('Invalid interval "%1$s".', $interval['period']));
 						break;
@@ -801,7 +946,7 @@ elseif ($valid_input && hasRequest('massupdate') && hasRequest('group_itemid'))
 						continue;
 					}
 
-					if (strpos($interval['schedule'], ';') !== false) {
+					if ($scheduling_interval_parser->parse($interval['schedule']) != CParser::PARSE_SUCCESS) {
 						$result = false;
 						info(_s('Invalid interval "%1$s".', $interval['schedule']));
 						break;
@@ -929,8 +1074,31 @@ elseif ($valid_input && hasRequest('massupdate') && hasRequest('group_itemid'))
 					'ipmi_sensor' => getRequest('ipmi_sensor'),
 					'applications' => $applications,
 					'status' => getRequest('status'),
-					'master_itemid' => getRequest('master_itemid')
+					'master_itemid' => getRequest('master_itemid'),
+					'url' =>  getRequest('url'),
+					'post_type' => getRequest('post_type'),
+					'posts' => getRequest('posts'),
+					'headers' => getRequest('headers', []),
+					'allow_traps' => getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF)
 				];
+
+				if ($item['headers']) {
+					$headers = [];
+
+					foreach ($item['headers']['name'] as $index => $key) {
+						if (array_key_exists($index, $item['headers']['value'])) {
+							$headers[$key] = $item['headers']['value'][$index];
+						}
+					}
+
+					// Ignore single row if it is empty.
+					if (count($headers) == 1 && $key === '' && $item['headers']['value'][$index] === '') {
+						$headers = [];
+					}
+
+					$item['headers'] = $headers;
+				}
+
 				if (hasRequest('preprocessing')) {
 					$preprocessing = getRequest('preprocessing');
 
@@ -957,11 +1125,8 @@ elseif ($valid_input && hasRequest('massupdate') && hasRequest('group_itemid'))
 
 					$item['preprocessing'] = $preprocessing;
 				}
-				foreach ($item as $key => $field) {
-					if ($field === null) {
-						unset($item[$key]);
-					}
-				}
+
+				$item = array_intersect_key($item, $visible);
 
 				$discovered_item = [];
 				if (hasRequest('status')) {
@@ -1121,40 +1286,32 @@ elseif (hasRequest('action') && getRequest('action') === 'item.massclearhistory'
 	show_messages($result, _('History cleared'), _('Cannot clear history'));
 }
 elseif (hasRequest('action') && getRequest('action') === 'item.massdelete' && hasRequest('group_itemid')) {
-	DBstart();
-
 	$group_itemid = getRequest('group_itemid');
 
-	$itemsToDelete = API::Item()->get([
-		'output' => ['key_', 'itemid'],
-		'selectHosts' => ['name'],
-		'itemids' => $group_itemid,
-		'preservekeys' => true
-	]);
-
 	$result = API::Item()->delete($group_itemid);
 
-	if ($result) {
-		foreach ($itemsToDelete as $item) {
-			$host = reset($item['hosts']);
-			add_audit(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_ITEM,
-				_('Item').' ['.$item['key_'].'] ['.$item['itemid'].'] '._('Host').' ['.$host['name'].']'
-			);
-		}
-	}
-
-	$result = DBend($result);
-
 	if ($result) {
 		uncheckTableRows(getRequest('hostid'));
 	}
 	show_messages($result, _('Items deleted'), _('Cannot delete items'));
 }
+elseif (hasRequest('action') && getRequest('action') === 'item.masscheck_now' && hasRequest('group_itemid')) {
+	$result = (bool) API::Task()->create([
+		'type' => ZBX_TM_TASK_CHECK_NOW,
+		'itemids' => getRequest('group_itemid')
+	]);
+
+	if ($result) {
+		uncheckTableRows(getRequest('hostid'));
+	}
+
+	show_messages($result, _('Request sent successfully'), _('Cannot send request'));
+}
 
 /*
  * Display
  */
-if (isset($_REQUEST['form']) && str_in_array($_REQUEST['form'], [_('Create item'), 'update', 'clone'])) {
+if (isset($_REQUEST['form']) && str_in_array($_REQUEST['form'], ['create', 'update', 'clone'])) {
 	$master_item_options = [];
 	$has_errors = false;
 
@@ -1165,9 +1322,12 @@ if (isset($_REQUEST['form']) && str_in_array($_REQUEST['form'], [_('Create item'
 				'snmpv3_securitylevel',	'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'logtimefmt', 'templateid',
 				'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey', 'privatekey',
 				'flags', 'interfaceid', 'port', 'description', 'inventory_link', 'lifetime', 'snmpv3_authprotocol',
-				'snmpv3_privprotocol', 'snmpv3_contextname', 'jmx_endpoint', 'master_itemid'
+				'snmpv3_privprotocol', 'snmpv3_contextname', 'jmx_endpoint', 'master_itemid', 'url', 'query_fields',
+				'timeout', 'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers',
+				'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
+				'verify_peer', 'verify_host', 'allow_traps'
 			],
-			'selectHosts' => ['status'],
+			'selectHosts' => ['status', 'name'],
 			'selectDiscoveryRule' => ['itemid', 'name'],
 			'selectPreprocessing' => ['type', 'params'],
 			'itemids' => getRequest('itemid')
@@ -1185,17 +1345,17 @@ if (isset($_REQUEST['form']) && str_in_array($_REQUEST['form'], [_('Create item'
 			$item['jmx_endpoint'] = ZBX_DEFAULT_JMX_ENDPOINT;
 		}
 
-		if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+		if (getRequest('type', $item['type']) == ITEM_TYPE_DEPENDENT) {
 			$master_item_options = [
 				'output' => ['itemid', 'type', 'hostid', 'name', 'key_'],
-				'itemids' => $item['master_itemid'],
+				'itemids' => getRequest('master_itemid', $item['master_itemid']),
 				'webitems' => true
 			];
 		}
 	}
 	else {
 		$hosts = API::Host()->get([
-			'output' => ['status'],
+			'output' => ['hostid', 'name', 'status'],
 			'hostids' => getRequest('hostid'),
 			'templated_hosts' => true
 		]);
@@ -1250,7 +1410,7 @@ elseif (((hasRequest('action') && getRequest('action') === 'item.massupdateform'
 	$data = [
 		'form' => getRequest('form'),
 		'action' => 'item.massupdateform',
-		'hostid' => getRequest('hostid'),
+		'hostid' => getRequest('hostid', 0),
 		'itemids' => getRequest('group_itemid', []),
 		'description' => getRequest('description', ''),
 		'delay' => getRequest('delay', ZBX_ITEM_DELAY_DEFAULT),
@@ -1285,12 +1445,34 @@ elseif (((hasRequest('action') && getRequest('action') === 'item.massupdateform'
 		'initial_item_type' => null,
 		'multiple_interface_types' => false,
 		'visible' => getRequest('visible', []),
-		'master_itemname' => getRequest('master_itemname', ''),
-		'master_itemid' => getRequest('master_itemid', 0)
+		'master_itemid' => getRequest('master_itemid', 0),
+		'url' =>  getRequest('url', ''),
+		'post_type' => getRequest('post_type', DB::getDefault('items', 'post_type')),
+		'posts' => getRequest('posts', ''),
+		'headers' => getRequest('headers', []),
+		'allow_traps' => getRequest('allow_traps', HTTPCHECK_ALLOW_TRAPS_OFF)
 	];
 
 	$data['displayApplications'] = true;
 	$data['displayInterfaces'] = true;
+	$data['displayMasteritems'] = true;
+
+	if ($data['headers']) {
+		$headers = [];
+
+		foreach ($data['headers']['name'] as $index => $key) {
+			if (array_key_exists($index, $data['headers']['value'])) {
+				$headers[] = [$key => $data['headers']['value'][$index]];
+			}
+		}
+
+		// Ignore single row if it is empty.
+		if (count($headers) == 1 && $key === '' && $data['headers']['value'][$index] === '') {
+			$headers = [];
+		}
+
+		$data['headers'] = $headers;
+	}
 
 	// hosts
 	$data['hosts'] = API::Host()->get([
@@ -1303,6 +1485,7 @@ elseif (((hasRequest('action') && getRequest('action') === 'item.massupdateform'
 	if ($hostCount > 1) {
 		$data['displayApplications'] = false;
 		$data['displayInterfaces'] = false;
+		$data['displayMasteritems'] = false;
 	}
 	else {
 		// get template count to display applications multiselect only for single template
@@ -1315,16 +1498,19 @@ elseif (((hasRequest('action') && getRequest('action') === 'item.massupdateform'
 		if ($templateCount != 0) {
 			$data['displayInterfaces'] = false;
 
-			if ($templateCount == 1 && !$data['hostid']) {
-				// if selected from filter without 'hostid'
+			if ($templateCount == 1 && $data['hostid'] == 0) {
+				// If selected from filter without 'hostid'.
 				$templates = reset($templates);
 				$data['hostid'] = $templates['templateid'];
 			}
 
-			// if items belong to single template and some belong to single host, don't display application multiselect
-			// and don't display application multiselect for multiple templates
+			/*
+			 * If items belong to single template and some belong to single host, don't display application multiselect
+			 * and don't display application multiselect for multiple templates.
+			 */
 			if ($hostCount == 1 && $templateCount == 1 || $templateCount > 1) {
 				$data['displayApplications'] = false;
+				$data['displayMasteritems'] = false;
 			}
 		}
 
@@ -1336,24 +1522,47 @@ elseif (((hasRequest('action') && getRequest('action') === 'item.massupdateform'
 				['field' => 'main', 'order' => ZBX_SORT_DOWN]
 			]);
 
-			// if selected from filter without 'hostid'
-			if (!$data['hostid']) {
+			// If selected from filter without 'hostid'.
+			if ($data['hostid'] == 0) {
 				$data['hostid'] = $data['hosts']['hostid'];
 			}
 
 			// set the initial chosen interface to one of the interfaces the items use
 			$items = API::Item()->get([
 				'itemids' => $data['itemids'],
-				'output' => ['itemid', 'type']
+				'output' => ['itemid', 'type', 'name']
 			]);
 			$usedInterfacesTypes = [];
+			$items_names = [];
 			foreach ($items as $item) {
 				$usedInterfacesTypes[$item['type']] = itemTypeInterface($item['type']);
+				$items_names[$item['itemid']] = $item['name'];
 			}
 			$initialItemType = min(array_keys($usedInterfacesTypes));
 			$data['type'] = (getRequest('type') !== null) ? ($data['type']) : $initialItemType;
 			$data['initial_item_type'] = $initialItemType;
 			$data['multiple_interface_types'] = (count(array_unique($usedInterfacesTypes)) > 1);
+			$data['items_names'] = $items_names;
+		}
+	}
+
+	if ($data['master_itemid'] != 0 && $data['displayMasteritems']) {
+		$master_items = API::Item()->get([
+			'output' => ['itemid', 'name'],
+			'selectHosts' => ['name'],
+			'itemids' => $data['master_itemid'],
+			'hostids' => $data['hostid'],
+			'webitems' => true
+		]);
+
+		if ($master_items) {
+			$data['master_itemname'] = $master_items[0]['name'];
+			$data['master_hostname'] = $master_items[0]['hosts'][0]['name'];
+		}
+		else {
+			$data['master_itemid'] = 0;
+			show_messages(false, '', _('No permissions to referred object or it does not exist!'));
+			$has_errors = true;
 		}
 	}
 
@@ -1489,9 +1698,9 @@ else {
 	}
 
 	/*
-	 * Trapper and SNMP trap items contain zeroes in "delay" field and, if no specific type is set, look in item types
-	 * other than trapper and SNMP trap that allow zeroes. For example, when a flexible interval is used. Since trapper
-	 * and SNMP trap items contain zeroes, but those zeroes should not be displayed, they cannot be filtered by entering
+	 * Trapper and SNMP trap items contain zeros in "delay" field and, if no specific type is set, look in item types
+	 * other than trapper and SNMP trap that allow zeros. For example, when a flexible interval is used. Since trapper
+	 * and SNMP trap items contain zeros, but those zeros should not be displayed, they cannot be filtered by entering
 	 * either zero or any other number in filter field.
 	 */
 	if (hasRequest('filter_delay')) {
@@ -1546,6 +1755,10 @@ else {
 			&& $_REQUEST['filter_templated_items'] != -1) {
 		$options['inherited'] = $_REQUEST['filter_templated_items'];
 	}
+	if (isset($_REQUEST['filter_discovery']) && !zbx_empty($_REQUEST['filter_discovery'])
+			&& $_REQUEST['filter_discovery'] != -1) {
+		$options['filter']['flags'] = $_REQUEST['filter_discovery'];
+	}
 	if (isset($_REQUEST['filter_with_triggers']) && !zbx_empty($_REQUEST['filter_with_triggers'])
 			&& $_REQUEST['filter_with_triggers'] != -1) {
 		$options['with_triggers'] = $_REQUEST['filter_with_triggers'];
@@ -1562,26 +1775,10 @@ else {
 		$data['items'] = [];
 	}
 
-	// set values for subfilters, if any of subfilters = false then item shouldnt be shown
+	$data['parent_templates'] = [];
+
+	// Set values for subfilters, if any of subfilters = false then item shouldn't be shown.
 	if ($data['items']) {
-		// fill template host
-		fillItemsWithChildTemplates($data['items']);
-
-		$dbHostItems = DBselect(
-			'SELECT i.itemid,h.name,h.hostid'.
-			' FROM hosts h,items i'.
-			' WHERE i.hostid=h.hostid'.
-				' AND '.dbConditionInt('i.itemid', zbx_objectValues($data['items'], 'templateid'))
-		);
-		while ($dbHostItem = DBfetch($dbHostItems)) {
-			foreach ($data['items'] as &$item) {
-				if ($item['templateid'] == $dbHostItem['itemid']) {
-					$item['template_host'] = $dbHostItem;
-				}
-			}
-			unset($item);
-		}
-
 		// resolve name macros
 		$data['items'] = expandItemNamesWithMasterItems($data['items'], 'items');
 
@@ -1635,6 +1832,9 @@ else {
 				'subfilter_templated_items' => empty($_REQUEST['subfilter_templated_items'])
 					|| ($item['templateid'] == 0 && uint_in_array(0, $_REQUEST['subfilter_templated_items'])
 					|| ($item['templateid'] > 0 && uint_in_array(1, $_REQUEST['subfilter_templated_items']))),
+				'subfilter_discovery' => empty($_REQUEST['subfilter_discovery'])
+					|| ($item['flags'] == ZBX_FLAG_DISCOVERY_NORMAL && uint_in_array(0, $_REQUEST['subfilter_discovery'])
+					|| ($item['flags'] == ZBX_FLAG_DISCOVERY_CREATED && uint_in_array(1, $_REQUEST['subfilter_discovery']))),
 				'subfilter_with_triggers' => empty($_REQUEST['subfilter_with_triggers'])
 					|| (count($item['triggers']) == 0 && uint_in_array(0, $_REQUEST['subfilter_with_triggers']))
 					|| (count($item['triggers']) > 0 && uint_in_array(1, $_REQUEST['subfilter_with_triggers'])),
@@ -1731,6 +1931,7 @@ else {
 	}
 
 	$data['paging'] = getPagingLine($data['items'], $sortOrder, new CUrl('items.php'));
+	$data['parent_templates'] = getItemParentTemplates($data['items'], ZBX_FLAG_DISCOVERY_NORMAL);
 
 	$itemTriggerIds = [];
 	foreach ($data['items'] as $item) {
@@ -1743,31 +1944,8 @@ else {
 		'selectFunctions' => API_OUTPUT_EXTEND,
 		'preservekeys' => true
 	]);
-	$data['triggerRealHosts'] = getParentHostsByTriggers($data['itemTriggers']);
 
-	// Select writable templates IDs.
-	$hostids = [];
-
-	foreach ($data['triggerRealHosts'] as $real_host) {
-		$hostids = array_merge($hostids, zbx_objectValues($real_host, 'hostid'));
-	}
-
-	foreach ($data['items'] as $item) {
-		if (array_key_exists('template_host', $item)) {
-			$hostids = array_merge($hostids, zbx_objectValues($item['template_host'], 'hostid'));
-		}
-	}
-
-	$data['writable_templates'] = [];
-
-	if ($hostids) {
-		$data['writable_templates'] = API::Template()->get([
-			'output' => ['templateid'],
-			'templateids' => array_keys(array_flip($hostids)),
-			'editable' => true,
-			'preservekeys' => true
-		]);
-	}
+	$data['trigger_parent_templates'] = getTriggerParentTemplates($data['itemTriggers'], ZBX_FLAG_DISCOVERY_NORMAL);
 
 	// determine, show or not column of errors
 	if (isset($hosts)) {
